MODULE p5zsed
   !!======================================================================
   !!                         ***  MODULE p5zsed  ***
   !! TOP :   PISCES Compute loss of organic matter in the sediments
   !!======================================================================
   !! History :   1.0  !  2004-03 (O. Aumont) Original code
   !!             2.0  !  2007-12 (C. Ethe, G. Madec)  F90
   !!             3.4  !  2011-06 (C. Ethe) USE of fldread
   !!             3.5  !  2012-07 (O. Aumont) improvment of river input of nutrients 
   !!             3.6  !  2015-05  (O. Aumont) PISCES quota
   !!----------------------------------------------------------------------
#if defined key_pisces_quota
   !!----------------------------------------------------------------------
   !!   'key_pisces_quota'     PISCES bio-model with variable stoichiometry
   !!----------------------------------------------------------------------
   !!   p5z_sed        :  Compute loss of organic matter in the sediments
   !!----------------------------------------------------------------------
   USE oce_trc         !  shared variables between ocean and passive tracers
   USE trc             !  passive tracers common variables 
   USE sms_pisces      !  PISCES Source Minus Sink variables
   USE p5zsink         !  vertical flux of particulate matter due to sinking
   USE p4zopt          !  optical model
   USE p5zlim          !  Co-limitations of differents nutrients
   USE p5zrem          !  Remineralisation of organic matter
   USE p4zsbc          !  External source of nutrients 
   USE p4zint          !  interpolation and computation of various fields
   USE iom             !  I/O manager
   USE prtctl_trc      !  print control for debugging

   IMPLICIT NONE
   PRIVATE

   PUBLIC   p5z_sed   
   PUBLIC   p5z_sed_alloc

   !! * Module variables
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:,:) :: nitrpot    !: Nitrogen fixation 
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:  ) :: sdenit     !: Nitrate reduction in the sediments
   REAL(wp) :: r1_rday                  !: inverse of rday

   !!* Substitution
#  include "top_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/TOP 3.3 , NEMO Consortium (2010)
   !! $Header:$ 
   !! Software governed by the CeCILL licence (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE p5z_sed( kt, knt )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE p5z_sed  **
      !!
      !! ** Purpose :   Compute loss of organic matter in the sediments. This
      !!              is by no way a sediment model. The loss is simply 
      !!              computed to balance the inout from rivers and dust
      !!
      !! ** Method  : - ???
      !!---------------------------------------------------------------------
      !
      INTEGER, INTENT(in) ::   kt, knt ! ocean time step
      INTEGER  ::   ji, jj, jk, ikt, jn
#if ! defined key_sed
      REAL(wp) ::   zsumsedsi, zsumsedpo4, zsumsedcal
      REAL(wp) ::   zrivalk, zrivsil, zrivno3
#endif
      REAL(wp) ::  zwflux, zfminus, zfplus
      REAL(wp) ::  zlim, zfact, zfactcal
      REAL(wp) ::  zo2, zno3, zflx, zpdenit, z1pdenit, zdenitt, zolimit
      REAL(wp) ::  zsiloss, zcaloss, zws3, zws4, zwsc, zdep
      REAL(wp) ::  zwstpoc, zwstpon, zwstpop
      REAL(wp) ::  ztrfer, ztrdp, zwdust, zlight, zmudia, ztemp
      REAL(wp) ::  xdiano3, xdianh4
      !
      CHARACTER (len=25) :: charout
      REAL(wp), POINTER, DIMENSION(:,:  ) :: zsidep, zwork1, zwork2, zwork3, zwork4
      REAL(wp), POINTER, DIMENSION(:,:  ) :: zdenit2d, zironice, zbureff
      REAL(wp), POINTER, DIMENSION(:,:  ) :: zwsbio3, zwsbio4, zwscal
      REAL(wp), POINTER, DIMENSION(:,:,:) :: ztrpo4, ztrdop, zirondep, zsoufer, zpdep
#if defined key_ligand
      REAL(wp) ::  zwssfep
      REAL(wp), POINTER, DIMENSION(:,:  ) :: zwsfep
#endif
      !!---------------------------------------------------------------------
      !
      IF( nn_timing == 1 )  CALL timing_start('p5z_sed')
      !
      IF( kt == nittrc000 .AND. knt == 1 )   r1_rday  = 1. / rday
      !
      ! Allocate temporary workspace
      CALL wrk_alloc( jpi, jpj, zdenit2d, zwork1, zwork2, zwork3, zwork4, zbureff )
      CALL wrk_alloc( jpi, jpj, zwsbio3, zwsbio4, zwscal )
      CALL wrk_alloc( jpi, jpj, jpk, ztrpo4, ztrdop, zsoufer )
#if defined key_ligand
      CALL wrk_alloc( jpi, jpj, zwsfep )
#endif


      zdenit2d(:,:) = 0.e0
      zbureff (:,:) = 0.e0
      zwork1  (:,:) = 0.e0
      zwork2  (:,:) = 0.e0
      zwork3  (:,:) = 0.e0
      zwork4  (:,:) = 0.e0

      ! Iron input/uptake due to sea ice : Crude parameterization based on Lancelot et al.
      ! ----------------------------------------------------
      IF( ln_ironice ) THEN  
         !                                              
         CALL wrk_alloc( jpi, jpj, zironice )
         !                                              
         DO jj = 1, jpj
            DO ji = 1, jpi
               zdep    = rfact2 / fse3t(ji,jj,1)
               zwflux  = fmmflx(ji,jj) / 1000._wp
               zfminus = MIN( 0._wp, -zwflux ) * trb(ji,jj,1,jpfer) * zdep
               zfplus  = MAX( 0._wp, -zwflux ) * icefeinput * zdep
               zironice(ji,jj) =  zfplus + zfminus
            END DO
         END DO
         !
         trb(:,:,1,jpfer) = trb(:,:,1,jpfer) + zironice(:,:) 
         !                                              
         IF( ln_diatrc .AND. lk_iomput .AND. knt == nrdttrc )   &
            &   CALL iom_put( "Ironice", zironice(:,:) * 1.e+3 * rfact2r * fse3t(:,:,1) * tmask(:,:,1) ) ! iron flux from ice
         CALL wrk_dealloc( jpi, jpj, zironice )
         !                                              
      ENDIF

      ! Add the external input of nutrients from dust deposition
      ! ----------------------------------------------------------
      IF( ln_dust ) THEN
         !                                              
         CALL wrk_alloc( jpi, jpj,      zsidep )
         CALL wrk_alloc( jpi, jpj, jpk, zpdep, zirondep      )
         !                                              ! Iron and Si deposition at the surface
         IF( ln_solub ) THEN
            zirondep(:,:,1) = solub(:,:) * dust(:,:) * mfrac * rfact2    &
            &        / fse3t(:,:,1) / 55.85 + 3.e-10 * r1_ryyss 
         ELSE
            zirondep(:,:,1) = dustsolub  * dust(:,:) * mfrac * rfact2    &
            &        / fse3t(:,:,1) / 55.85 + 3.e-10 * r1_ryyss 
         ENDIF
         zsidep(:,:) = 8.8 * 0.075 * dust(:,:) * mfrac * rfact2 / fse3t(:,:,1) / 28.1 
         zpdep (:,:,1) = 0.2 * 0.023 * dust(:,:) * mfrac * rfact2 / fse3t(:,:,1) / 31. / po4r 
         !                                              ! Iron solubilization of particles in the water column
         !                                              ! dust in kg/m2/s ---> 1/55.85 to put in mol/Fe ;  wdust in m/j
         zwdust = 0.03 * rday / ( wdust * 55.85 ) / ( 270. * rday )
         DO jk = 2, jpkm1
            zirondep(:,:,jk) = dust(:,:) * mfrac * zwdust * rfact2  &
            &   * EXP( -fsdept(:,:,jk) / 540. )
            zpdep(:,:,jk) = dust(:,:) * mfrac * 0.023 * zwdust * rfact2  &
            &   * EXP( -fsdept(:,:,jk) / 540. )
         END DO
         !                                              ! Iron solubilization of particles in the water column
         trb(:,:,:,jppo4) = trb(:,:,:,jppo4) + zpdep   (:,:,:)
         trb(:,:,1,jpsil) = trb(:,:,1,jpsil) + zsidep  (:,:)
         trb(:,:,:,jpfer) = trb(:,:,:,jpfer) + zirondep(:,:,:) 
         !                                              
         IF( ln_diatrc ) THEN
            zfact = 1.e+3 * rfact2r
            IF( lk_iomput ) THEN
               IF( knt == nrdttrc ) THEN
                  CALL iom_put( "Irondep", zirondep(:,:,1) * zfact * fse3t(:,:,1) * tmask(:,:,1) ) ! surface downward dust depo of iron
                  CALL iom_put( "pdust"  , dust(:,:) / ( wdust * rday )  * tmask(:,:,1) ) ! dust concentration at surface
               ENDIF
            ELSE
               trc2d(:,:,jp_pcs0_2d + 11) = zirondep(:,:,1) * zfact * fse3t(:,:,1) * tmask(:,:,1)
            ENDIF
         ENDIF
         CALL wrk_dealloc( jpi, jpj,      zsidep )
         CALL wrk_dealloc( jpi, jpj, jpk, zpdep, zirondep      )
         !                                              
      ENDIF
     
      ! Add the external input of nutrients from river
      ! ----------------------------------------------------------
      IF( ln_river ) THEN
         trb(:,:,1,jppo4) = trb(:,:,1,jppo4) + rivdip(:,:) * rfact2
         trb(:,:,1,jpdop) = trb(:,:,1,jpdop) + rivdop(:,:) * rfact2
         trb(:,:,1,jpno3) = trb(:,:,1,jpno3) + rivdin(:,:) * rfact2
         trb(:,:,1,jpdon) = trb(:,:,1,jpdon) + rivdon(:,:) * rfact2
         trb(:,:,1,jpfer) = trb(:,:,1,jpfer) + rivdic(:,:) * 5.e-5 * rfact2
         trb(:,:,1,jpsil) = trb(:,:,1,jpsil) + rivdsi(:,:) * rfact2
         trb(:,:,1,jpdic) = trb(:,:,1,jpdic) + rivdic(:,:) * rfact2
         trb(:,:,1,jpdoc) = trb(:,:,1,jpdoc) + rivdoc(:,:) * rfact2
         trb(:,:,1,jptal) = trb(:,:,1,jptal) + ( rivalk(:,:) - rno3 * rivdin(:,:) ) * rfact2
      ENDIF

      ! Add the external input of nutrients from nitrogen deposition
      ! ----------------------------------------------------------
      IF( ln_ndepo ) THEN
         trb(:,:,1,jpno3) = trb(:,:,1,jpno3) + nitdep(:,:) * rfact2
         trb(:,:,1,jptal) = trb(:,:,1,jptal) - rno3 * nitdep(:,:) * rfact2
      ENDIF

      ! Add the external input of iron from sediment mobilization
      ! ------------------------------------------------------
      IF( ln_ironsed ) THEN
         trb(:,:,:,jpfer) = trb(:,:,:,jpfer) + ironsed(:,:,:) * rfact2
#if defined key_ligand
         trn(:,:,:,jpfep) = trn(:,:,:,jpfep) + (ironsed(:,:,:) * fep_rats ) * rfact2
#endif
         !
         IF( ln_diatrc .AND. lk_iomput .AND. knt == nrdttrc )   &
            &   CALL iom_put( "Ironsed", ironsed(:,:,:) * 1.e+3 * tmask(:,:,:) ) ! iron inputs from sediments
      ENDIF

      ! Add the external input of iron from hydrothermal vents
      ! ------------------------------------------------------
      IF( ln_hydrofe ) THEN
         trb(:,:,:,jpfer) = trb(:,:,:,jpfer) + hydrofe(:,:,:) * rfact2
#if defined key_ligand
         trn(:,:,:,jpfep) = trn(:,:,:,jpfep) + ( hydrofe(:,:,:) * fep_rath ) * rfact2
         trn(:,:,:,jplgw) = trn(:,:,:,jplgw) + ( hydrofe(:,:,:) * hydrolig ) * rfact2
#endif
         !
         IF( ln_diatrc .AND. lk_iomput .AND. knt == nrdttrc )   &
            &   CALL iom_put( "HYDR", hydrofe(:,:,:) * 1.e+3 * tmask(:,:,:) ) ! hydrothermal iron input
      ENDIF

      ! OA: Warning, the following part is necessary, especially with Kriest
      ! to avoid CFL problems above the sediments
      ! --------------------------------------------------------------------
      DO jj = 1, jpj
         DO ji = 1, jpi
            ikt  = mbkt(ji,jj)
            zdep = fse3t(ji,jj,ikt) / xstep
            zwsbio4(ji,jj) = MIN( 0.99 * zdep, wsbio4(ji,jj,ikt) )
            zwscal (ji,jj) = MIN( 0.99 * zdep, wscal (ji,jj,ikt) )
            zwsbio3(ji,jj) = MIN( 0.99 * zdep, wsbio3(ji,jj,ikt) )
#if defined key_ligand
            zwsfep(ji,jj)  = MIN( 0.99 * zdep, wsfep(ji,jj,ikt)  )
#endif
         END DO
      END DO

#if ! defined key_sed
      ! Computation of the sediment denitrification proportion: The metamodel from midlleburg (2006) is being used
      ! Computation of the fraction of organic matter that is permanently buried from Dunne's model
      ! -------------------------------------------------------
      DO jj = 1, jpj
         DO ji = 1, jpi
           IF( tmask(ji,jj,1) == 1 ) THEN
              ikt = mbkt(ji,jj)
              zflx = (  trb(ji,jj,ikt,jpgoc) * zwsbio4(ji,jj)   &
                &     + trb(ji,jj,ikt,jppoc) * zwsbio3(ji,jj) )  * 1E3 * 1E6 / 1E4
              zflx  = LOG10( MAX( 1E-3, zflx ) )
              zo2   = LOG10( MAX( 10. , trb(ji,jj,ikt,jpoxy) * 1E6 ) )
              zno3  = LOG10( MAX( 1.  , trb(ji,jj,ikt,jpno3) * 1E6 * rno3 ) )
              zdep  = LOG10( fsdepw(ji,jj,ikt+1) )
              zdenit2d(ji,jj) = -2.2567 - 1.185 * zflx - 0.221 * zflx**2 - 0.3995 * zno3 * zo2 + 1.25 * zno3    &
              &                + 0.4721 * zo2 - 0.0996 * zdep + 0.4256 * zflx * zo2
              zdenit2d(ji,jj) = 10.0**( zdenit2d(ji,jj) )
              !
              zflx = (  trb(ji,jj,ikt,jpgoc) * zwsbio4(ji,jj)   &
                &     + trb(ji,jj,ikt,jppoc) * zwsbio3(ji,jj) ) * 1E6
              zbureff(ji,jj) = 0.013 + 0.53 * zflx**2 / ( 7.0 + zflx )**2
           ENDIF
         END DO
      END DO 

      ! Loss of biogenic silicon, Caco3 organic carbon in the sediments. 
      ! First, the total loss is computed.
      ! The factor for calcite comes from the alkalinity effect
      ! -------------------------------------------------------------
      DO jj = 1, jpj
         DO ji = 1, jpi
            IF( tmask(ji,jj,1) == 1 ) THEN
               ikt = mbkt(ji,jj) 
               zwork1(ji,jj) = trb(ji,jj,ikt,jpgsi) * zwsbio4(ji,jj)
               zwork2(ji,jj) = trb(ji,jj,ikt,jpgoc) * zwsbio4(ji,jj) + trb(ji,jj,ikt,jppoc) * zwsbio3(ji,jj) 
               ! For calcite, burial efficiency is made a function of saturation
               zfactcal      = MIN( excess(ji,jj,ikt), 0.2 )
               zfactcal      = MIN( 1., 1.3 * ( 0.2 - zfactcal ) / ( 0.4 - zfactcal ) )
               zwork3(ji,jj) = trb(ji,jj,ikt,jpcal) * zwscal(ji,jj) * 2.e0 * zfactcal
            ENDIF
         END DO
      END DO
      zsumsedsi  = glob_sum( zwork1(:,:) * e1e2t(:,:) ) * r1_rday
      zsumsedpo4 = glob_sum( zwork2(:,:) * e1e2t(:,:) ) * r1_rday
      zsumsedcal = glob_sum( zwork3(:,:) * e1e2t(:,:) ) * r1_rday
#endif

      ! This loss is scaled at each bottom grid cell for equilibrating the total budget of silica in the ocean.
      ! Thus, the amount of silica lost in the sediments equal the supply at the surface (dust+rivers)
      ! ------------------------------------------------------
#if ! defined key_sed
      zrivsil =  MAX(0., 1._wp - ( sumdepsi + rivdsiinput * r1_ryyss ) / ( zsumsedsi + rtrn ) )
#endif

      DO jj = 1, jpj
         DO ji = 1, jpi
            ikt  = mbkt(ji,jj)
            zdep = xstep / fse3t(ji,jj,ikt)
            zws4 = zwsbio4(ji,jj) * zdep
            zwsc = zwscal (ji,jj) * zdep
            zsiloss = trb(ji,jj,ikt,jpgsi) * zwsc
            zcaloss = trb(ji,jj,ikt,jpcal) * zwsc
            !
            trb(ji,jj,ikt,jpgsi) = trb(ji,jj,ikt,jpgsi) - zsiloss
            trb(ji,jj,ikt,jpcal) = trb(ji,jj,ikt,jpcal) - zcaloss
#if ! defined key_sed
            trb(ji,jj,ikt,jpsil) = trb(ji,jj,ikt,jpsil) + zsiloss * zrivsil 
            zfactcal = MIN( excess(ji,jj,ikt), 0.2 )
            zfactcal = MIN( 1., 1.3 * ( 0.2 - zfactcal ) / ( 0.4 - zfactcal ) )
            zrivalk  =  1._wp - ( rivalkinput * r1_ryyss ) * zfactcal / ( zsumsedcal + rtrn )
            trb(ji,jj,ikt,jptal) =  trb(ji,jj,ikt,jptal) + zcaloss * zrivalk * 2.0
            trb(ji,jj,ikt,jpdic) =  trb(ji,jj,ikt,jpdic) + zcaloss * zrivalk
#endif
         END DO
      END DO


      DO jj = 1, jpj
         DO ji = 1, jpi
            ikt     = mbkt(ji,jj)
            zdep    = xstep / fse3t(ji,jj,ikt)
            zws4 = zwsbio4(ji,jj) * zdep
            zws3 = zwsbio3(ji,jj) * zdep
#if defined key_ligand
            zwssfep = zwsfep(ji,jj) * zdep
#endif
            zrivno3 = 1. - zbureff(ji,jj)
            trb(ji,jj,ikt,jpgoc) = trb(ji,jj,ikt,jpgoc) - trb(ji,jj,ikt,jpgoc) * zws4
            trb(ji,jj,ikt,jppoc) = trb(ji,jj,ikt,jppoc) - trb(ji,jj,ikt,jppoc) * zws3
            trb(ji,jj,ikt,jpgon) = trb(ji,jj,ikt,jpgon) - trb(ji,jj,ikt,jpgon) * zws4
            trb(ji,jj,ikt,jppon) = trb(ji,jj,ikt,jppon) - trb(ji,jj,ikt,jppon) * zws3
            trb(ji,jj,ikt,jpgop) = trb(ji,jj,ikt,jpgop) - trb(ji,jj,ikt,jpgop) * zws4
            trb(ji,jj,ikt,jppop) = trb(ji,jj,ikt,jppop) - trb(ji,jj,ikt,jppop) * zws3
            trb(ji,jj,ikt,jpbfe) = trb(ji,jj,ikt,jpbfe) - trb(ji,jj,ikt,jpbfe) * zws4
            trb(ji,jj,ikt,jpsfe) = trb(ji,jj,ikt,jpsfe) - trb(ji,jj,ikt,jpsfe) * zws3
            zwstpoc              = trb(ji,jj,ikt,jpgoc) * zws4 + trb(ji,jj,ikt,jppoc) * zws3 
            zwstpop              = trb(ji,jj,ikt,jpgop) * zws4 + trb(ji,jj,ikt,jppop) * zws3
            zwstpon              = trb(ji,jj,ikt,jpgon) * zws4 + trb(ji,jj,ikt,jppon) * zws3
#   if defined key_ligand
            trn(ji,jj,ikt,jpfep) = trn(ji,jj,ikt,jpfep) - trn(ji,jj,ikt,jpfep) * zwssfep
#   endif

#if ! defined key_sed
            ! The 0.5 factor in zpdenit and zdenitt is to avoid negative NO3 concentration after both denitrification
            ! in the sediments and just above the sediments. Not very clever, but simpliest option.
            zpdenit  = MIN( 0.5 * ( trb(ji,jj,ikt,jpno3) - rtrn ) / rdenit, zdenit2d(ji,jj) * zwstpoc * zrivno3 )
            zpdenit = MAX( 0., zpdenit )
            z1pdenit = zwstpoc * zrivno3 - zpdenit
            zolimit = MIN( ( trb(ji,jj,ikt,jpoxy) - rtrn ) / o2ut, z1pdenit * ( 1.- nitrfac(ji,jj,ikt) ) )
            zdenitt = MIN(  0.5 * ( trb(ji,jj,ikt,jpno3) - rtrn ) / rdenit, z1pdenit * nitrfac(ji,jj,ikt) )
            zolimit = MAX( 0., zolimit )
            zdenitt = MAX( 0., zdenitt )
            trb(ji,jj,ikt,jpdoc) = trb(ji,jj,ikt,jpdoc) + z1pdenit - zolimit - zdenitt
            trb(ji,jj,ikt,jpdon) = trb(ji,jj,ikt,jpdon) + (z1pdenit - zolimit - zdenitt) * zwstpon / (zwstpoc + rtrn)
            trb(ji,jj,ikt,jpdop) = trb(ji,jj,ikt,jpdop) + (z1pdenit - zolimit - zdenitt) * zwstpop / (zwstpoc + rtrn)
            trb(ji,jj,ikt,jppo4) = trb(ji,jj,ikt,jppo4) + (zpdenit + zolimit + zdenitt) * zwstpop / (zwstpoc + rtrn)
            trb(ji,jj,ikt,jpnh4) = trb(ji,jj,ikt,jpnh4) + (zpdenit + zolimit + zdenitt) * zwstpon / (zwstpoc + rtrn)
            trb(ji,jj,ikt,jpno3) = trb(ji,jj,ikt,jpno3) - rdenit * (zpdenit + zdenitt)
            trb(ji,jj,ikt,jpoxy) = trb(ji,jj,ikt,jpoxy) - zolimit * o2ut
            trb(ji,jj,ikt,jptal) = trb(ji,jj,ikt,jptal) + rno3 * (zolimit + zpdenit + zdenitt ) * zwstpon / (zwstpoc + rtrn)   &
            &   + rno3 * rdenit * (zpdenit + zdenitt)
            trb(ji,jj,ikt,jpdic) = trb(ji,jj,ikt,jpdic) + zpdenit + zolimit + zdenitt
            zwork4(ji,jj) = rdenit * zpdenit * fse3t(ji,jj,ikt)
#endif
         END DO
      END DO

      ! Nitrogen fixation process
      ! Small source iron from particulate inorganic iron
      !-----------------------------------
      DO jk = 1, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi
               !                      ! Potential nitrogen fixation dependant on temperature and iron
               ztemp = tsn(ji,jj,jk,jp_tem)
               zmudia = max(0.,-0.001096*ztemp**2 + 0.057*ztemp -0.637) * 7.625
               !       Potential nitrogen fixation dependant on
               !       temperature and iron
               xdianh4 = trb(ji,jj,jk,jpnh4) / ( concnnh4 + trb(ji,jj,jk,jpnh4) )
               xdiano3 = trb(ji,jj,jk,jpno3) / ( concnno3 + trb(ji,jj,jk,jpno3) ) * (1. - xdianh4)
               zlim = ( 1.- xdiano3 - xdianh4 )
               IF( zlim <= 0.1 )   zlim = 0.01
               zfact = zlim * rfact2
               ztrfer = biron(ji,jj,jk) / ( concfediaz + biron(ji,jj,jk) )
               ztrpo4(ji,jj,jk) = trb(ji,jj,jk,jppo4) / ( 1E-6 + trb(ji,jj,jk,jppo4) )
               ztrdop(ji,jj,jk) = trb(ji,jj,jk,jpdop) / ( 1E-6 + trb(ji,jj,jk,jpdop) ) * (1. - ztrpo4(ji,jj,jk))
               ztrdp = ztrpo4(ji,jj,jk) + ztrdop(ji,jj,jk)
               zlight =  ( 1.- EXP( -etot(ji,jj,jk) / diazolight ) ) * (1. - fr_i(ji,jj)) 
               nitrpot(ji,jj,jk) =  zmudia * r1_rday * zfact * MIN( ztrfer, ztrdp ) * zlight
               zsoufer(ji,jj,jk) = zlight * 2E-11 / (2E-11 + biron(ji,jj,jk))
            END DO
         END DO
      END DO

      ! Nitrogen change due to nitrogen fixation
      ! ----------------------------------------
      DO jk = 1, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi
               zfact = nitrpot(ji,jj,jk) * nitrfix
               trb(ji,jj,jk,jpnh4) = trb(ji,jj,jk,jpnh4) + zfact / 3.0
               trb(ji,jj,jk,jptal) = trb(ji,jj,jk,jptal) + rno3 * zfact / 3.0
               trb(ji,jj,jk,jppo4) = trb(ji,jj,jk,jppo4) - 16.0 / 46.0 * zfact * ( 1.0 - 1.0 / 3.0 ) &
               &                     * ztrpo4(ji,jj,jk) / (ztrpo4(ji,jj,jk) + ztrdop(ji,jj,jk) + rtrn)
               trb(ji,jj,jk,jpdon) = trb(ji,jj,jk,jpdon) + zfact * 1.0 / 3.0
               trb(ji,jj,jk,jpdoc) = trb(ji,jj,jk,jpdoc) + zfact * 1.0 / 3.0
               trb(ji,jj,jk,jpdop) = trb(ji,jj,jk,jpdop) + 16.0 / 46.0 * zfact / 3.0  &
               &                     - 16.0 / 46.0 * zfact * ztrdop(ji,jj,jk)   &
               &                     / (ztrpo4(ji,jj,jk) + ztrdop(ji,jj,jk) + rtrn)
               trb(ji,jj,jk,jppoc) = trb(ji,jj,jk,jppoc) + zfact * 1.0 / 3.0 * 2.0 / 3.0
               trb(ji,jj,jk,jppon) = trb(ji,jj,jk,jppon) + zfact * 1.0 / 3.0 * 2.0 /3.0
               trb(ji,jj,jk,jppop) = trb(ji,jj,jk,jppop) + 16.0 / 46.0 * zfact * 1.0 / 3.0 * 2.0 /3.0
               trb(ji,jj,jk,jpgoc) = trb(ji,jj,jk,jpgoc) + zfact * 1.0 / 3.0 * 1.0 / 3.0
               trb(ji,jj,jk,jpgon) = trb(ji,jj,jk,jpgon) + zfact * 1.0 / 3.0 * 1.0 /3.0
               trb(ji,jj,jk,jpgop) = trb(ji,jj,jk,jpgop) + 16.0 / 46.0 * zfact * 1.0 / 3.0 * 1.0 /3.0
               trb(ji,jj,jk,jpoxy) = trb(ji,jj,jk,jpoxy) + ( o2ut + o2nit ) * zfact * 2.0 / 3.0 + o2nit * zfact / 3.0
               trb(ji,jj,jk,jpfer) = trb(ji,jj,jk,jpfer) - 30E-6 * zfact * 1.0 / 3.0 
               trb(ji,jj,jk,jpsfe) = trb(ji,jj,jk,jpsfe) + 30E-6 * zfact * 1.0 / 3.0 * 2.0 / 3.0
               trb(ji,jj,jk,jpbfe) = trb(ji,jj,jk,jpbfe) + 30E-6 * zfact * 1.0 / 3.0 * 1.0 / 3.0
               trb(ji,jj,jk,jpfer) = trb(ji,jj,jk,jpfer) + 0.002 * 4E-10 * zsoufer(ji,jj,jk) * rfact2 / rday
           END DO
         END DO 
      END DO
      !
      IF( lk_iomput ) THEN
         IF( knt == nrdttrc ) THEN
            zfact = 1.e+3 * rfact2r * rno3  !  conversion from molC/l/kt  to molN/m3/s
            IF( iom_use("Nfix"   ) ) CALL iom_put( "Nfix", nitrpot(:,:,:) * nitrfix * zfact * tmask(:,:,:) )  ! nitrogen fixation 
            IF( iom_use("INTNFIX") ) THEN   ! nitrogen fixation rate in ocean ( vertically integrated )
               zwork1(:,:) = 0.
               DO jk = 1, jpkm1
                 zwork1(:,:) = zwork1(:,:) + nitrpot(:,:,jk) * nitrfix * zfact * fse3t(:,:,jk) * tmask(:,:,jk)
               ENDDO
               CALL iom_put( "INTNFIX" , zwork1 )
            ENDIF
         ENDIF
      ELSE
         IF( ln_diatrc )  &
            &  trc2d(:,:,jp_pcs0_2d + 12) = nitrpot(:,:,1) * nitrfix * rno3 * 1.e+3 * rfact2r * fse3t(:,:,1) * tmask(:,:,1)
      ENDIF
      !
      IF(ln_ctl) THEN  ! print mean trends (USEd for debugging)
         WRITE(charout, fmt="('sed ')")
         CALL prt_ctl_trc_info(charout)
      ENDIF
      !
      CALL wrk_dealloc( jpi, jpj, zdenit2d, zwork1, zwork2, zwork3, zwork4, zbureff )
      CALL wrk_dealloc( jpi, jpj, zwsbio3, zwsbio4, zwscal )
      CALL wrk_dealloc( jpi, jpj, jpk, ztrpo4, ztrdop, zsoufer )
#if defined key_ligand
      CALL wrk_dealloc( jpi, jpj, zwsfep )
#endif
      !
      IF( nn_timing == 1 )  CALL timing_stop('p5z_sed')
      !
 9100  FORMAT(i8,3f10.5)
      !
   END SUBROUTINE p5z_sed

   INTEGER FUNCTION p5z_sed_alloc()
      !!----------------------------------------------------------------------
      !!                     ***  ROUTINE p4z_sed_alloc  ***
      !!----------------------------------------------------------------------
      ALLOCATE( nitrpot(jpi,jpj,jpk), sdenit(jpi,jpj), STAT=p5z_sed_alloc )
      !
      IF( p5z_sed_alloc /= 0 )   CALL ctl_warn('p5z_sed_alloc: failed to allocate arrays')
      !
   END FUNCTION p5z_sed_alloc

#else
   !!======================================================================
   !!  Dummy module :                                   No PISCES bio-model
   !!======================================================================
CONTAINS
   SUBROUTINE p5z_sed                         ! Empty routine
   END SUBROUTINE p5z_sed
#endif 

   !!======================================================================
END MODULE  p5zsed
